#! /usr/bin/perl

# This program will back up the locally modified and unknown files in
# a list of CVS working directories, the list being either passed on
# the command line, or from the contents of a file.  Each directory
# will be backed up into its own archive, which will be named after
# the directory and login ID of the user who owns it.  (eg:
# "cvs-utils.karlheg.afio") I recommend that you rotate these archives
# prior to each round of backups, using something like `savelog'.

# If you are using remote CVS, you'll have to make sure that `cvs
# login' gets run before this script starts, or it will fail silently.

# Copyright 1998, Karl M. Hegbloom <karlheg@debian.org>
# Copyright is hereby assigned to Software in the Public Interest, Inc.

# When submitting patches, please use the:
#   -u --ignore-matching-lines='\$Id.*\$'
# switches to `diff', `rcsdiff', or `cvs diff'.
# $Id: backup-cvs-work,v 1.2 1998/08/19 18:09:37 root Exp $

# This program is distributed without Warrantee.  For the details of
# the licencing terms, please read the GNU General Public Licence.
# (See: "/usr/doc/copyright/GPL")

# This is a good default on a Debian GNU/Linux setup.
$backupdir = '/var/backups/cvs-work-backups';

foreach ( @ARGV) {
  /^--backupdir=(\S+)/ && {$backupdir = $1, next};
  /^--dirlistfile=(\S+)/ && {$dirlistfile = $1, last};
  unshift (@workdirs, $_);
}

unless ( $dirlistfile or scalar (@workdirs) ) {
  print STDERR << "EOF";
Usage: $0 [--backupdir=BACKUPS_GO_HERE] dir [[dir] ...]
   Or: $0 [--backupdir=BACKUPS_GO_HERE] --dirlistfile=FILE_LISTING_DIRS

 Where DIR is a checked out CVS working directory with an accessable
 repository.  The directories listed should be separated with
 newlines. \`--backupdir\' defaults to \`$backupdir\'.
EOF
  exit 1;
}

unless ( -d $backupdir ) {
  print STDOUT "Createing \`$backupdir\'.\n";
  unless ( mkdir ($backupdir, 0644)) {
    die "Cannot mkdir().";
  }
}

if ( $dirlistfile ) {
  open (DIRLIST, "< $dirlistfile") or die "Cannot open() `$dirlistfile'.";
  @workdirs = <DIRLIST>;
  close (DIRLIST);
  chop @workdirs;
}

foreach $workdir (@workdirs) {

  unless ( $workdir =~ m!.*/$!o ) {
    $workdir .= "/";
  }

  unless ( chdir ($workdir) ) {
    warn "Cannot `chdir ($workdir)'.";
    next;
  }
  
  unless ( open (CVS, "cvs status 2>&1 |") ) {
    warn "Cannot popen `cvs status' in `$workdir'.";
    next;
  }

  $dir = '';
  while ( <CVS> ) {
    /^cvs (?:status|server): Examining (.*)$/o && {$dir = "$1/", next};
    /^File: (\S+)\s+Status: Locally Modified/o && {push (@backup, "$dir$1"), next};
    /^File: (\S+)\s+Status: Unknown/o && {push (@backup, "$dir$1"), next};
    /^\? (\S+)/o && { &push_question ($1), next};
  }
  close (CVS);
  
  unless ( chdir ($backupdir) ) {
    die "Cannot chdir() to the backup directory, `$backupdir'.";
  }

  $dirname = $workdir;
  $dirname =~ s!(.*)/$!\1!;
  $dirname =~ m!^.*?([^/]+)$!;
  $user = (getpwuid ( (stat ($workdir))[4] ) )[0];
  $volume = "$1.$user.afio";

  print STDOUT
    "Backing up ${user}\'s \`$workdir\' changed files in \`$volume\'.\n",
    "------------------------------------------------------------------------------\n";
  unless ( open (AFIO, "| /bin/afio -oZG9 $volume") ) {
     warn "Cannot popen afio for `$volume' in `$backupdir'.";
     next;
  }
  foreach $file ( @backup ) {
    print AFIO $workdir . $file . "\n";
    print STDOUT $workdir . $file . "\n";
  }
  close ( AFIO );
  @backup = ();
  print STDOUT
    "------------------------------------------------------------------------------\n\n";
}

sub push_question {
  my ($name) = @_;
  if ( -d $name ) {
    open (FIND, "find $name/ -depth -print |") || die "Cannot find.";
    while (<FIND>) {
      chomp;
      push (@backup, $_);
    }
    close (FIND);
  }
  else {
    push (@backup, "$name");
  }
}
